@@ -37,6 +37,8 @@ use vars qw(@hooks);
 use strict;
 use Carp;
 
+sub CLONE_SKIP { 1 }
+
 sub new { shift-> create(@_) }
 
 sub CREATE
@@ -212,10 +214,10 @@ sub get_notify_sub
 			return $sub, $self;
 		}
 		my $method = "on_" . lc $note;
-		return ( $sub, $self) if $sub = $self-> can( $method, 0);
+		return ( $sub, $self) if $sub = $self-> can( $method);
 	} else {
 		my ( $sub, $method) = ( undef, "on_" . lc $note);
-		return ( $sub, $self) if $sub = $self-> can( $method, 0);
+		return ( $sub, $self) if $sub = $self-> can( $method);
 		my ( $referer, $sub2, $id) = $self-> get_notification( $note, ($rnt & nt::FluxReverse) ? -1 : 0);
 		if ( defined $referer) {
 			return ( $sub, $referer, $self) if $referer != $self;
@@ -289,10 +291,15 @@ sub profile_default
 	$def-> {name} = 'Clipboard';
 	return $def;
 }
+
 sub text
 { 
 	if ($#_) {
-		$_[0]-> store( 'Text',  $_[1]);
+		my ( $self, $text ) = @_;
+		$self-> open;
+		$self-> clear;
+		$::application-> notify( 'CopyText', $self, $text );
+		$self-> close;
 	} else {
 		my $text;
 		$::application-> notify( 'PasteText', $_[0], \$text);
@@ -300,12 +307,26 @@ sub text
 	}
 }
 
-sub image { $#_ ? $_[0]-> store( 'Image', $_[1]) : return $_[0]-> fetch('Image') }
+sub image
+{ 
+	if ($#_) {
+		my ( $self, $image ) = @_;
+		$self-> open;
+		$self-> clear;
+		$::application-> notify( 'CopyImage', $self, $image);
+		$self-> close;
+	} else {
+		my $image;
+		$::application-> notify( 'PasteImage', $_[0], \$image);
+		return $image;
+	}
+}
 
 # class Drawable
 package Prima::Drawable;
 use vars qw(@ISA);
 @ISA = qw(Prima::Component);
+use Prima::Bidi qw(is_bidi);
 
 my $sysData = Prima::Application-> get_system_info;
 
@@ -379,7 +400,7 @@ sub stretch_image {
 		@_[3,4], $_[5]-> size, 
 		defined ($_[6]) ? $_[6] : $_[0]-> rop
 	) if $_[5]
-} 
+}
 
 sub rect3d
 {
@@ -387,7 +408,9 @@ sub rect3d
 	my $c = $self-> color;
 	if ( defined $backColor)
 	{
-		if ( $backColor == cl::Back) {
+		if ( ref $backColor ) {
+			$self-> gradient_bar($x + $width, $y + $width, $x1 - $width, $y1 - $width, $backColor);
+		} elsif ( $backColor == cl::Back) {
 			$self-> clear( $x + $width, $y + $width, $x1 - $width, $y1 - $width);
 		} else {
 			$self-> color( $backColor);
@@ -482,6 +505,10 @@ sub draw_text
 
 	return 0 unless scalar @lines;
 
+	if (($flags & dt::BidiText) && $Prima::Bidi::available) {
+		$_ = Prima::Bidi::visual($_) for @lines;
+	}
+
 	my @clipSave;
 	my $fh = $canvas-> font-> height +
 		(( $flags & dt::UseExternalLeading) ? 
@@ -553,6 +580,184 @@ sub draw_text
 	return $retVal;
 }
 
+sub prelight_color
+{
+	my ( $self, $color, $coeff ) = @_;
+	$coeff //= 1.05;
+	return 0 if $coeff <= 0;
+	$color = $self->map_color($color) if $color & cl::SysFlag;
+	if (( $color == 0xffffff && $coeff > 1) || ($color == 0 && $coeff < 1)) {
+		$coeff = 1/$coeff;
+	}
+	$coeff = ($coeff - 1) * 256;
+	my @channels = map { $_ & 0xff } ($color >> 16), ($color >> 8), $color;
+	my ( $below, $above ) = (0,0);
+	for (@channels) {
+		my $amp = ( 256 - $_ ) / 8;
+		$amp -= $amp if $coeff < 0;
+		$_ += $coeff + $amp;
+		$_ = 255 if $_ > 255;
+		$_ = 0   if $_ < 0;
+	}
+	return ( $channels[0] << 16 ) | ( $channels[1] << 8 ) | $channels[2];
+}
+
+sub gradient_polyline_to_points
+{
+	my ($self, $p) = @_;
+	my @map;
+	for ( my $i = 0; $i < @$p - 2; $i+=2) {
+		my ($x1,$y1,$x2,$y2) = @$p[$i..$i+3];
+		$x1 = 0 if $x1 < 0;
+		my $dx = $x2 - $x1;
+		if ( $dx > 0 ) {
+			my $dy = ($y2 - $y1) / $dx;
+			my $y = $y1;
+			for ( my $x = $x1; $x <= $x2; $x++) {
+				$map[$x] = $y;
+				$y += $dy;
+			}
+		} else {
+			$map[$x1] = $y1;
+		}
+	}
+	return \@map;
+}
+
+sub gradient_realize3d
+{
+	my ( $self, $breadth, $request) = @_;
+
+	my ($offsets, $points);
+
+	unless ( $points = $request->{points}) {
+		my @spline = (0,0);
+		if ( my $s = $request->{spline} ) {
+			push @spline, map { $_ * $breadth } @$s;
+		}
+		if ( my $s = $request->{poly} ) {
+			push @spline, map { $_ * $breadth } @$s;
+		}
+		push @spline, $breadth, $breadth;
+		my $polyline = ( @spline > 4 && $request->{spline} ) ? $self-> render_spline( \@spline ) : \@spline;
+		$points = $self-> gradient_polyline_to_points($polyline);
+	}
+
+	unless ($offsets = $request->{offsets}) {
+		my @o;
+		my $n = scalar(@{$request->{palette}}) - 1;
+		my $d = 0;
+		for ( my $i = 0; $i < $n; $i++) {
+			$d += 1/$n;
+			push @o, $d;
+		}
+		push @o, 1;
+		$offsets = \@o;
+	}
+	
+	$request->{points}  = $points;
+	$request->{offsets} = $offsets;
+
+	return $self-> gradient_calculate(
+		$request->{palette},
+		[ map { $_ * $breadth } @$offsets ],
+		sub { $points->[shift] }
+	);
+}
+
+sub gradient_calculate_single
+{
+	my ( $self, $breadth, $start_color, $end_color, $function, $offset ) = @_;
+
+	return [] if $breadth <= 0;
+
+	$offset //= 0;
+	$start_color = $self->map_color($start_color) if $start_color & cl::SysFlag;
+	$end_color   = $self->map_color($end_color)   if $end_color   & cl::SysFlag;
+	my @start = map { $_ & 0xff } ($start_color >> 16), ($start_color >> 8), $start_color;
+	my @end   = map { $_ & 0xff } ($end_color   >> 16), ($end_color   >> 8), $end_color;
+	my @color = @start;
+	
+	my @delta = map { ( $end[$_] - $start[$_] ) / $breadth } 0..2;
+
+	my $last_color = $start_color;
+	my $color      = $start_color;
+	my $width      = 0;
+	my @ret;
+	for ( my $i = 0; $i < $breadth; $i++) {
+		if ( $last_color != $color ) {
+			push @ret, $last_color = $color, $width;
+			$width = 0;
+		}
+
+		my @c;
+		my $j = $function ? $function->( $offset + $i ) - $offset : $i;
+		for ( 0..2 ) {
+			$color[$_] = $start[$_] + $j * $delta[$_] for 0..2;
+			$c[$_] = int($color[$_] + .5);
+			$c[$_] = 0xff if $c[$_] > 0xff;
+			$c[$_] = 0    if $c[$_] < 0;
+		}
+		$color = ( $c[0] << 16 ) | ( $c[1] << 8 ) | $c[2];
+		$width++;
+	}
+
+	return @ret, $color, $width;
+}
+
+sub gradient_calculate
+{
+	my ( $self, $palette, $offsets, $function ) = @_;
+	my @ret;
+	my $last_offset = 0;
+	$offsets = [ $offsets ] unless ref $offsets;
+	for ( my $i = 0; $i < @$offsets; $i++) {
+		my $breadth = $offsets->[$i] - $last_offset;
+		push @ret, $self-> gradient_calculate_single( $breadth, $palette->[$i], $palette->[$i+1], $function, $last_offset);
+		$last_offset = $offsets->[$i];
+	}
+	return \@ret;
+}
+
+sub gradient_bar
+{
+	my ( $self, $x1, $y1, $x2, $y2, $request ) = @_;
+
+	($x1,$x2)=($x2,$x1) if $x1 > $x2;
+	($y1,$y2)=($y2,$y1) if $y1 > $y2;
+
+	my $gradient = $request->{gradient} //= $self-> gradient_realize3d( 
+		$request->{vertical} ? 
+			$x2 - $x1 + 1 :
+			$y2 - $y1 + 1, 
+		$request
+	);
+
+	my @bar        = ($x1,$y1,$x2,$y2);
+	my ($ptr1,$ptr2) = $request->{vertical} ? (0,2) : (1,3);
+	my $max          = $bar[$ptr2];
+	for ( my $i = 0; $i < @$gradient; $i+=2) {
+		$bar[$ptr2] = $bar[$ptr1] + $gradient->[$i+1] - 1;
+		$self->color( $gradient->[$i]);
+		$self->bar( @bar );
+		$bar[$ptr1] = $bar[$ptr2] + 1;
+		last if $bar[$ptr1] > $max;
+	}
+	if ( $bar[$ptr1] <= $max ) {
+		$bar[$ptr2] = $max;
+		$self->bar(@bar);
+	}
+}
+
+sub text_out_bidi
+{
+	if ( $Prima::Bidi::enabled && is_bidi $_[1] ) {
+		return shift->text_out( Prima::Bidi::visual(shift), @_);
+	} else {
+		return shift->text_out(@_);
+	}
+}
+
 # class Image
 package Prima::Image;
 use vars qw( @ISA);
@@ -631,6 +836,13 @@ sub colormap
 	}
 }
 
+sub clone
+{
+	my $i = shift->dup;
+	$i->set(@_);
+	return $i;
+}
+
 # class Icon
 package Prima::Icon;
 use vars qw( @ISA);
@@ -649,6 +861,24 @@ sub profile_default
 	return $def;
 }
 
+sub mirror
+{
+        my ($self, $vertically) = @_;
+        my ($xor, $and) = $self->split;
+        $and->preserveType(1);
+        $_->mirror($vertically) for $xor, $and;
+        $self->combine($xor, $and);
+}
+
+sub rotate
+{
+        my ($self, $degrees) = @_;
+        my ($xor, $and) = $self->split;
+        $and->preserveType(1);
+        $_->rotate($degrees) for $xor, $and;
+        $self->combine($xor, $and);
+}
+
 # class DeviceBitmap
 package Prima::DeviceBitmap;
 use vars qw( @ISA);
@@ -1160,16 +1390,26 @@ sub defocus     { $_[0]-> focused(0); }
 
 # Tk namespace and syntax compatibility
 
+sub __tk_dash_map
+{
+	my %ret;
+	my %hash = @_;
+	while ( my ( $k, $v ) = each %hash ) {
+		$k =~ s/^-//;
+		$ret{$k} = $v;
+	}
+	return %ret;
+}
+
 sub pack { 
 	my $self = shift;
-	# XXX regexp kills valid undefs   
-	$self-> packInfo( { grep { defined } map { /^(?:-(\D.*))|(.*)$/; } @_ });
+	$self-> packInfo( { __tk_dash_map(@_) });
 	$self-> geometry( gt::Pack);
 }
 
 sub place { 
 	my $self = shift;
-	$self-> placeInfo( { grep { defined } map { /^(?:-(\D.*))|(.*)$/; } @_ });
+	$self-> placeInfo( { __tk_dash_map(@_) });
 	$self-> geometry( gt::Place);
 }
 
@@ -1204,12 +1444,17 @@ sub rect_bevel
 		push @c3d, $back, 0x404040;
 	}
 
+	if ( my $g = $opt{gradient} ) {
+		for (@{$g->{palette} // []}) {
+			$_ = $self->map_color($_) if $_ & cl::SysFlag;
+		}
+	}
+
 	my $hw = int( $width / 2);
-	$canvas-> rect3d( $x, $y, $x1, $y1, $hw, @c3d[2,3], $fill);
+	$canvas-> rect3d( $x, $y, $x1, $y1, $hw, @c3d[2,3], $opt{gradient} // $fill);
 	$canvas-> rect3d( $x + $hw, $y + $hw, $x1 - $hw, $y1 - $hw, $width - $hw, @c3d[0,1]);
 }
 
-
 # class Window
 package Prima::Window;
 use vars qw(@ISA);
@@ -1237,6 +1482,7 @@ sub profile_default
 		clipOwner             => 0,
 		growMode              => gm::DontCare,
 		icon                  => 0,
+		mainWindow            => 0,
 		menu                  => undef,
 		menuItems             => undef,
 		menuColor             => cl::NormalText,
@@ -1354,7 +1600,17 @@ package Prima::MainWindow;
 use vars qw(@ISA);
 @ISA = qw(Prima::Window);
 
-sub on_create  { $::main_window = $_[0]; }
+sub profile_default
+{
+	my $def = $_[ 0]-> SUPER::profile_default;
+	my %prf = (
+		mainWindow => 1,
+	);
+	@$def{keys %prf} = values %prf;
+	return $def;
+}
+
+sub on_create  { $::main_window = $_[0] }
 sub on_destroy { $::application-> close; undef $::main_window }
 
 # class MenuItem
@@ -1495,7 +1751,7 @@ sub on_paint
 	my $fh = $canvas-> font-> height;
 	my ( $x, $y) = ( 3, $size[1] - 1 - $fh);
 	for ( @ln) {
-		$canvas-> text_out( $_, $x, $y);
+		$canvas-> text_out_bidi( $_, $x, $y);
 		$y -= $fh;
 	}
 }
@@ -1516,16 +1772,20 @@ use vars qw(@ISA @startupNotifications);
 {
 my %RNT = (
 	%{Prima::Widget-> notification_types()},
+	CopyText    => nt::Action,
 	PasteText   => nt::Action,
+	CopyImage   => nt::Action,
+	PasteImage  => nt::Action,
 );
 
 sub notification_types { return \%RNT; }
 }
+	
+my $unix = Prima::Application-> get_system_info-> {apc} == apc::Unix;
 
 sub profile_default
 {
 	my $def  = $_[ 0]-> SUPER::profile_default;
-	my $unix = Prima::Application-> get_system_info-> {apc} == apc::Unix;
 	my %prf = (
 		autoClose      => 0,
 		pointerType    => cr::Arrow,
@@ -1588,6 +1848,24 @@ sub setup
 	}
 	$_-> ($self) for @startupNotifications;
 	undef @startupNotifications;
+
+	# setup image cliboard transfer routines specific to gtk
+	if ( $unix ) {
+		my %weights = (
+			bmp  => 4,  # bmp is independent on codecs
+			png  => 3,  # png is lossless
+			tiff => 2,  # tiff is usually lossless
+		);
+		my %codecs  = map { lc($_-> {fileShortType})  => $_ } @{Prima::Image-> codecs};
+		$_->{weight} = $weights{ lc($_-> {fileShortType}) } || 1 for values %codecs;
+		my @codecs = map { {
+			mime => "image/$_",
+			id   => $codecs{$_}->{codecID},
+		} } sort { $codecs{$b}->{weight} <=> $codecs{$a}->{weight} } keys %codecs;
+		my $clipboard = $self-> Clipboard;
+		$clipboard-> register_format($_->{mime}) for @codecs;
+		$self-> {GTKImageClipboardFormats} = \@codecs;
+	}
 }
 
 sub get_printer
@@ -1628,6 +1906,24 @@ sub open_help
 	return $self-> {HelpClass}-> open($link);
 }
 
+sub on_copytext
+{
+	my ( $self, $clipboard, $text ) = @_;
+	$clipboard-> store( 'Text',  $text);
+}
+
+sub on_copyimage
+{
+	my ( $self, $clipboard, $image) = @_;
+	$clipboard-> store( 'Image',  $image);
+	if ( my $formats = $self-> {GTKImageClipboardFormats} ) {
+		my ($bmp, $data, $handle) = ($formats->[0], '');
+		if (open( $handle, '>', \$data) and $image->save($handle, codecID => $bmp->{id})) {
+			$clipboard->store($bmp->{mime}, $data);
+		}
+	}
+}
+
 sub on_pastetext
 {
 	my ( $self, $clipboard, $ref) = @_;
@@ -1638,6 +1934,31 @@ sub on_pastetext
 	undef;
 }
 
+sub on_pasteimage
+{
+	my ( $self, $clipboard, $ref) = @_;
+	$$ref = $clipboard-> fetch( 'Image');
+	return if defined $$ref;
+
+	my $codecs = $self-> {GTKImageClipboardFormats};
+	return unless $codecs;
+
+	my %formats = map { $_ => 1 } $clipboard-> get_formats;
+	my @codecs  = grep { $formats{$_->{mime}} } @$codecs;
+	return unless @codecs;
+
+	my $data = $clipboard-> fetch($codecs[0]->{mime});
+	return unless defined $data;
+
+	my $handle;
+	open( $handle, '<', \$data) or return;
+
+	local $@;
+	$$ref = Prima::Image-> load($handle, loadExtras => 1 );
+
+	undef;
+}
+
 1;
 
 __DATA__
