@@ -46,7 +46,8 @@ use Prima::IntUtils;
 use Prima::Utils;
 use Cwd;
 
-package ci;
+package 
+    ci;
 
 BEGIN {
 eval 'use constant Grid => 1 + MaxId;' unless exists $ci::{Grid};
@@ -54,7 +55,7 @@ eval 'use constant Grid => 1 + MaxId;' unless exists $ci::{Grid};
 
 package Prima::AbstractListViewer;
 use vars qw(@ISA);
-@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller);
+@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller Prima::ListBoxUtils);
 
 use Prima::Classes;
 
@@ -75,6 +76,7 @@ sub profile_default
 {
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
+		align          => ta::Left,
 		autoHeight     => 1,
 		autoHScroll    => 1,
 		autoVScroll    => 1,
@@ -94,6 +96,9 @@ sub profile_default
 		offset         => 0,
 		topItem        => 0,
 		scaleChildren  => 0,
+		scrollBarClass => 'Prima::ScrollBar',
+		hScrollBarProfile=>{},
+		vScrollBarProfile=>{},
 		selectable     => 1,
 		selectedItems  => [],
 		vertical       => 1,
@@ -139,17 +144,18 @@ sub init
 		itemWidth offset multiColumn count autoHeight multiSelect 
 		extendedSelect borderWidth dragable ))
 		{ $self-> {$_} = 0; }
-	for ( qw( drawGrid itemHeight integralWidth integralHeight vertical))
+	for ( qw( drawGrid itemHeight integralWidth integralHeight vertical align))
 		{ $self-> {$_} = 1; }
 	$self-> {selectedItems} = {};
 	my %profile = $self-> SUPER::init(@_);
 	$self-> setup_indents;
 	$self-> {selectedItems} = {} unless $profile{multiSelect};
+	$self->{$_} = $profile{$_} for qw(scrollBarClass hScrollBarProfile vScrollBarProfile);
 	for ( qw( 
 		autoHScroll autoVScroll gridColor hScroll vScroll offset multiColumn 
 		itemHeight autoHeight itemWidth multiSelect extendedSelect integralHeight 
 		integralWidth focusedItem topItem selectedItems borderWidth dragable 
-		vertical drawGrid))
+		vertical drawGrid align))
 		{ $self-> $_( $profile{ $_}); }
 	$self-> reset;
 	$self-> reset_scrolls;
@@ -299,13 +305,14 @@ sub on_paint
 						(( $self-> {focusedItem} == $item) ? 1 : 0);
 					my $foc = ( $foci == $item) ? $focusedState : 0;
 					$foc = 1 if $item == 0 && $self-> {unfocVeil};
+					my $prelight = (defined($self->{prelight}) && ($self->{prelight} == $item)) ? 1 : 0;
 
 					push( @paintArray, [
 						$item,          # item number
 						$itemRect[0], $itemRect[1],
 						$itemRect[2]-1, $itemRect[3]-1,
-						$sel, $foc,     # selected and focused states
-						$j              # column
+						$sel, $foc, $prelight,  # selected and focused states
+						$j,             # column
 					]);
 					$item += $di;
 				}
@@ -326,13 +333,14 @@ sub on_paint
 					(( $foci == $item) ? 1 : 0);
 				my $foc = ( $foci == $item) ? $focusedState : 0;
 				$foc = 1 if $item == 0 && $self-> {unfocVeil};
+				my $prelight = (defined($self->{prelight}) && ($self->{prelight} == $item)) ? 1 : 0;
 
 				push( @paintArray, [
 					$item,      # item number
 					$itemRect[0] - $self-> {offset}, $itemRect[1],  # logic rect
 					$itemRect[2], $itemRect[3],                     #
-					$sel, $foc, # selected and focused state
-					0 #column
+					$sel, $foc, $prelight, # selected and focused state
+					0, #column,
 				]);
 				$item++;
 			}
@@ -600,7 +608,25 @@ sub on_mouseclick
 sub on_mousemove
 {
 	my ( $self, $mod, $x, $y) = @_;
-	return unless defined $self-> {mouseTransaction};
+	unless (defined $self-> {mouseTransaction}) {
+		if ( $self-> enabled ) {
+			my @a = $self-> get_active_area;
+			my $prelight;
+			if ( $y >= $a[1] && $y < $a[3] && $x >= $a[0] && $x < $a[2]) {
+				my ($item, $aux) = $self-> point2item( $x, $y);
+				$prelight = ($item >= 0) ? $item : undef unless defined $aux;
+			}
+			if ( ( $self->{prelight} // -1 ) != ( $prelight // -1 )) {
+				my @redraw = (
+					$self->{prelight} // (),
+					$prelight // ()
+				);
+				$self->{prelight} = $prelight;
+				$self->redraw_items( @redraw );
+			}
+		}
+		return;
+	}
 	
 	my $bw = $self-> { borderWidth};
 	my ($item, $aux) = $self-> point2item( $x, $y);
@@ -684,6 +710,13 @@ sub on_mouseup
 	$self-> notify(q(DragItem), @dragnotify) if @dragnotify;
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	my $prelight = delete $self->{prelight};
+	$self-> redraw_items( $prelight ) if defined $prelight;
+}
+
 sub on_mousewheel
 {
 	my ( $self, $mod, $x, $y, $z) = @_;
@@ -888,6 +921,14 @@ sub set_auto_height
 	$self-> {autoHeight} = $auto;
 }
 
+sub set_align
+{
+	my ( $self, $align) = @_;
+
+	$self-> {align} = $align;
+	$self-> repaint;
+}
+
 sub reset_indents
 {
 	my ( $self) = @_;
@@ -1294,7 +1335,7 @@ sub HScroll_Change
 
 #sub on_drawitem
 #{
-#	my ($self, $canvas, $itemIndex, $x, $y, $x2, $y2, $selected, $focused) = @_;
+#	my ($self, $canvas, $itemIndex, $x, $y, $x2, $y2, $selected, $focused, $prelight, $column) = @_;
 #}
 
 #sub on_selectitem
@@ -1308,6 +1349,7 @@ sub HScroll_Change
 #}
 
 sub autoHeight    {($#_)?$_[0]-> set_auto_height    ($_[1]):return $_[0]-> {autoHeight}     }
+sub align         {($#_)?$_[0]-> set_align          ($_[1]):return $_[0]-> {align}          }
 sub count         {($#_)?$_[0]-> set_count          ($_[1]):return $_[0]-> {count}          }
 sub extendedSelect{($#_)?$_[0]-> set_extended_select($_[1]):return $_[0]-> {extendedSelect} }
 sub drawGrid      {($#_)?$_[0]-> set_draw_grid      ($_[1]):return $_[0]-> {drawGrid}       }
@@ -1360,11 +1402,22 @@ sub draw_text_items
 {
 	my ( $self, $canvas, $first, $last, $step, $x, $y, $textShift, $clipRect) = @_;
 	my ($i,$j);
+	my ($dx,$iw) = (0);
+	if ( $self->{align} != ta::Left ) {
+		my @a = $self->item2rect( $first );
+		$iw = $a[2] - $a[0];
+		$iw = $self->{itemWidth} if $iw < $self->{itemWidth};
+	}
 	for ( $i = $first, $j = 1; $i <= $last; $i += $step, $j++) {
-		next if $self-> get_item_width( $i) + 
-			$self-> {offset} + $x + 1 < $clipRect-> [0];
-		$canvas-> text_out( $self-> get_item_text( $i), 
-			$x, $y + $textShift - $j * $self-> {itemHeight} + 1
+		my $width = $self-> get_item_width( $i);
+		next if $width + $self-> {offset} + $x + 1 < $clipRect-> [0];
+		if ( $self->{align} == ta::Center) {
+			$dx = ($iw > $width) ? ($iw - $width) / 2 : 0;
+		} elsif ( $self->{align} == ta::Right) {
+			$dx = ($iw > $width) ? $iw - $width : 0;
+		}
+		$canvas-> text_out_bidi( $self-> get_item_text( $i), 
+			$x + $dx, $y + $textShift - $j * $self-> {itemHeight} + 1
 		);
 	}
 }
@@ -1393,21 +1446,26 @@ sub std_draw_text_items
 		push ( @colContainer, [])
 	};
 	for ( $i = 0; $i < scalar @_; $i++) {
-		push ( @{$colContainer[ $_[$i]-> [7]]}, $_[$i]);
+		push ( @{$colContainer[ $_[$i]-> [8]]}, $_[$i]);
 		$drawVeilFoc = $i if $_[$i]-> [6];
 	}
 	my ( $lc, $lbc) = @clrs[0,1];
 	for ( @colContainer) {
 		my @normals;
 		my @selected;
+		my @prelight;
 		my ( $lastNormal, $lastSelected) = (undef, undef);
-		my $isSelected = 0;
 		# sorting items in single column
 		{ $_ = [ sort { $$a[0]<=>$$b[0] } @$_]; }
 		# calculating conjoint bars
 		for ( $i = 0; $i < scalar @$_; $i++) {
-			my ( $itemIndex, $x, $y, $x2, $y2, $selected, $focusedItem) = @{$$_[$i]};
-			if ( $selected) {
+			my ( $itemIndex, $x, $y, $x2, $y2, $selected, $focusedItem, $prelighted) = @{$$_[$i]};
+			if ( $prelighted ) {
+				push ( @prelight, [ 
+					$x, $y, $x2, $y2, 
+					$$_[$i]-> [0], $$_[$i]-> [0], $selected ? 3 : 2,
+				]);
+			} elsif ( $selected) {
 				if ( 
 					defined $lastSelected && 
 					( $y2 + 1 == $lastSelected)
@@ -1421,7 +1479,6 @@ sub std_draw_text_items
 					]);
 				}
 				$lastSelected = $y;
-				$isSelected = 1;
 			} else {
 				if ( 
 					defined $lastNormal && 
@@ -1439,17 +1496,24 @@ sub std_draw_text_items
 				$lastNormal = $y;
 			}
 		}
-		for ( @selected) { push ( @normals, $_); }
 		# draw items
 
-		for ( @normals) {
+		for ( @normals, @selected, @prelight) {
 			my ( $x, $y, $x2, $y2, $first, $last, $selected) = @$_;
-			my $c = $clrs[ $selected ? 3 : 1];
+			my $c;
+			my $prelight;
+			if ($selected & 2) {
+				$selected -= 2;
+				$prelight = 1;
+			}
+
+			$c = $clrs[ $selected ? 3 : 1];
 			if ( $c != $lbc) {
 				$canvas-> backColor( $c);
 				$lbc = $c;
 			}
-			$canvas-> clear( $x, $y, $x2, $y2);
+			
+			$self-> draw_item_background( $canvas, $x, $y, $x2, $y2, $prelight);
 
 			$c = $clrs[ $selected ? 2 : 0];
 			if ( $c != $lc) {
@@ -2091,7 +2155,7 @@ See L<DrawItem> for parameters description.
 
 Called by C<std_draw_text_items> to draw sequence of text items with 
 indices from FIRST to LAST, by STEP, on CANVAS, starting at point X, Y, and
-incrementing the vertical position with OFFSET. CLIP_RECT is a reference
+incrementing the horizontal position with OFFSET. CLIP_RECT is a reference
 to array of four integers with inclusive-inclusive coordinates of the active 
 clipping rectangle.
 
@@ -2197,13 +2261,13 @@ Called when the user finishes the drag of an item
 from OLD_INDEX to NEW_INDEX position. The default action
 rearranges the item list in accord with the dragging action.
 
-=item DrawItem CANVAS, INDEX, X1, Y1, X2, Y2, SELECTED, FOCUSED
+=item DrawItem CANVAS, INDEX, X1, Y1, X2, Y2, SELECTED, FOCUSED, PRELIGHT, COLUMN
 
 Called when an INDEXth item is to be drawn on CANVAS. 
 X1, Y1, X2, Y2 designate the item rectangle in widget coordinates,
-where the item is to be drawn. SELECTED and FOCUSED are boolean
+where the item is to be drawn. SELECTED, FOCUSED, and PRELIGHT are boolean
 flags, if the item must be drawn correspondingly in selected and
-focused states.
+focused states, with or without the prelight effect.
 
 =item MeasureItem INDEX, REF
 
