@@ -39,7 +39,7 @@ use Prima qw(Classes IntUtils StdBitmap);
 
 package Prima::OutlineViewer;
 use vars qw(@ISA @images @imageSize);
-@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller);
+@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller Prima::ListBoxUtils);
 
 use constant DATA     => 0;
 use constant DOWN     => 1;
@@ -90,6 +90,9 @@ sub profile_default
 		topItem        => 0,
 		offset         => 0,
 		scaleChildren  => 0,
+		scrollBarClass => 'Prima::ScrollBar',
+		hScrollBarProfile=>{},
+		vScrollBarProfile=>{},
 		selectable     => 1,
 		showItemHint   => 1,
 		vScroll        => 1,
@@ -155,6 +158,7 @@ sub init
 	$self-> {items}      = [];
 	my %profile = $self-> SUPER::init(@_);
 	$self-> setup_indents;
+	$self->{$_} = $profile{$_} for qw(scrollBarClass hScrollBarProfile vScrollBarProfile);
 	for ( qw( autoHScroll autoVScroll hScroll vScroll offset itemHeight autoHeight borderWidth 
 		indent items focusedItem topItem showItemHint dragable multiSelect extendedSelect))
 		{ $self-> $_( $profile{ $_}); }
@@ -337,6 +341,7 @@ sub on_paint
 	my ($array, $idx, $lim, $level) = ([['root'],$self-> {items}], 0, scalar @{$self-> {items}}, 0);
 	my @stack;
 	my $position = 0;
+	my $prelight = $self->{prelight} // -1;
 
 # preparing stack
 	$i = int(( $timin + 1) / STACK_FRAME) * STACK_FRAME - 1; 
@@ -380,7 +385,7 @@ sub on_paint
 				if $position >= $timin;
 				$lines[ $level] = undef;
 			} elsif ( $position > 0) {
-			# first and last
+				# first and last
 				$canvas-> bar(
 					$l, $firstY - $ih * ( $position - 0.5),
 					$l, $firstY - $ih * ( $position + 0.5))
@@ -397,8 +402,10 @@ sub on_paint
 			push ( @texts, [ $node, $l + $indent * 1.5, $lineY,
 				$l + $indent * 1.5 + $node-> [WIDTH] - 1, $lineY + $ih - 1,
 				$position, 
-						$self-> {multiSelect} ? $node-> [SELECTED] : ($foc == $position),
-					$foc == $position]);
+				$self-> {multiSelect} ? $node-> [SELECTED] : ($foc == $position),
+				$foc == $position,
+				$prelight == $position,
+			]);
 			$lineY -= $ih;
 		}
 		last if $position >= $lastItem;
@@ -567,8 +574,8 @@ sub makehint
 	my $ofs = ( $lev + 2.5) * $self-> {indent} - $self-> {offset} + $self-> {indents}-> [0];
 
 	if ( $w + $ofs <= $a[2]) {
-	$self-> makehint(0);
-	return;
+		$self-> makehint(0);
+		return;
 	}
 
 	$self-> {unsuccessfullId} = undef;
@@ -638,7 +645,7 @@ sub on_mousemove
 	my ( $self, $mod, $x, $y) = @_;
 	my @size = $self-> size;
 	my @a    = $self-> get_active_area( 0, @size);
-	if ( !defined $self-> {mouseTransaction} && $self-> {showItemHint}) {
+	if ( !defined $self-> {mouseTransaction} && $self->enabled) {
 		my $item   = $self-> point2item( $y, $size[1]);
 		my ( $rec, $lev) = $self-> get_item( $item);
 		if ( 
@@ -646,14 +653,30 @@ sub on_mousemove
 			( $x < -$self-> {offset} + ($lev + 2) * $self-> {indent} + $self-> {indents}-> [0])
 		) {
 			$self-> makehint( 0);
-			return;
-		}
-		if (( $y >= $a[3]) || ( $y <= $a[1] + $self-> {itemHeight} / 2)) {
+		} elsif (( $y >= $a[3]) || ( $y <= $a[1] + $self-> {itemHeight} / 2)) {
 			$self-> makehint( 0);
-			return;
+		} else {
+			$y = $a[3] - $y; 
+			$item = $self-> {topItem} + int( $y / $self-> {itemHeight}); 
 		}
-		$y = $a[3] - $y;
-		$self-> makehint( 1, $self-> {topItem} + int( $y / $self-> {itemHeight}));
+
+		if ( $self-> {showItemHint} ) {
+			if ( defined $item ) {
+				$self-> makehint( 1, $self-> {topItem} + int( $y / $self-> {itemHeight}));
+			} else {
+				$self-> makehint( 0);
+			}
+		}
+
+		if (( $self->{prelight} // -1) != ( $item // -1)) {
+			my @redraw = (
+				$self->{prelight} // (),
+				$item // ()
+			);
+			$self->{prelight} = $item;
+			$self->redraw_items( @redraw );
+		}
+
 		return;
 	}
 	my $item = $self-> point2item( $y, $size[1]);
@@ -725,6 +748,13 @@ sub on_mousewheel
 	$self-> topItem( $newTop > $maxTop ? $maxTop : $newTop);
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	my $prelight = delete $self->{prelight};
+	$self-> redraw_items( $prelight ) if defined $prelight;
+}
+
 sub on_enable  { $_[0]-> repaint; }
 sub on_disable { $_[0]-> repaint; }
 
@@ -987,6 +1017,23 @@ sub update_tree
 	$self-> offset( $self-> {offset});
 }
 
+sub redraw_items
+{
+	my ( $self, @items ) = @_;
+	my @a  = $self-> get_active_area;
+	my $ih = $self-> {itemHeight};
+
+	my %match;
+	for my $i ( @items ) {
+		next unless $i >= 0 && $i >= $self-> {topItem} && $i <= $self-> {topItem} + $self-> {rows};
+		next if $match{$i}++;
+		$i -= $self->{topItem};
+		$self-> invalidate_rect( 
+			$a[0], $a[3] - ( $i + 1) * $ih,
+			$a[2], $a[3] - $i * $ih
+		);
+	}
+}
 
 sub draw_items
 {
@@ -1039,23 +1086,7 @@ sub set_focused_item
 	my @a  = $self-> get_active_area;
 	my $ih = $self-> {itemHeight};
 	my $lastItem = $self-> {topItem} + $self-> {rows};
-	
-	$self-> invalidate_rect( 
-		$a[0], $a[3] - ( $oldFoc - $self-> {topItem} + 1) * $ih,
-		$a[2], $a[3] - ( $oldFoc - $self-> {topItem}) * $ih
-	) if 
-		$oldFoc >= 0 && 
-		$oldFoc != $foc && 
-		$oldFoc >= $self-> {topItem} && 
-		$oldFoc <= $self-> {topItem} + $self-> {rows};
-		
-	$self-> invalidate_rect( 
-		$a[0], $a[3] - ( $foc - $self-> {topItem} + 1) * $ih,
-		$a[2], $a[3] - ( $foc - $self-> {topItem}) * $ih
-	) if 
-		$foc >= 0 && 
-		$foc >= $self-> {topItem} && 
-		$foc <= $self-> {topItem} + $self-> {rows};
+	$self->redraw_items($oldFoc, $foc);
 }
 
 sub set_indent
@@ -1398,7 +1429,7 @@ sub get_index_width
 
 sub on_drawitem
 {
-#	my ( $self, $canvas, $node, $left, $bottom, $right, $top, $position, $selected, $focused) = @_;
+#	my ( $self, $canvas, $node, $left, $bottom, $right, $top, $position, $selected, $focused, $prelight) = @_;
 }
 
 sub on_measureitem
@@ -1613,16 +1644,20 @@ sub draw_items
 {
 	my ($self, $canvas, $paintStruc) = @_;
 	for ( @$paintStruc) {
-		my ( $node, $left, $bottom, $right, $top, $position, $selected, $focused) = @$_;
-		if ( $selected) {
+		my ( $node, $left, $bottom, $right, $top, $position, $selected, $focused, $prelight) = @$_;
+		if ( $selected || $prelight) {
+			my $bc = $self->backColor;
+			$self-> draw_item_background($canvas, 
+				$left, $bottom, $right, $top, $prelight,
+				$selected ? $self-> hiliteBackColor : $self-> backColor
+			);
+			$self->backColor($bc);
 			my $c = $canvas-> color;
-			$canvas-> color( $self-> hiliteBackColor);
-			$canvas-> bar( $left, $bottom, $right, $top);
-			$canvas-> color( $self-> hiliteColor);
-			$canvas-> text_out( $node-> [0], $left, $bottom);
+			$canvas-> color( $selected ? $self-> hiliteColor : $c );
+			$canvas-> text_out_bidi( $node-> [0], $left, $bottom);
 			$canvas-> color( $c);
 		} else {
-			$canvas-> text_out( $node-> [0], $left, $bottom);
+			$canvas-> text_out_bidi( $node-> [0], $left, $bottom);
 		}
 		$canvas-> rect_focus( $left, $bottom, $right, $top) if $focused;
 	}
@@ -1648,16 +1683,21 @@ sub draw_items
 {
 	my ($self, $canvas, $paintStruc) = @_;
 	for ( @$paintStruc) {
-		my ( $node, $left, $bottom, $right, $top, $position, $selected, $focused) = @$_;
-		if ( $selected) {
-			my $c = $canvas-> color;
-			$canvas-> color( $self-> hiliteBackColor);
-			$canvas-> bar( $left, $bottom, $right, $top);
-			$canvas-> color( $self-> hiliteColor);
-			$canvas-> text_out( $node-> [0]-> [0], $left, $bottom);
+		my ( $node, $left, $bottom, $right, $top, $position, $selected, $focused, $prelight) = @$_;
+		if ( $selected || $prelight ) {
+			my $bc = $self->backColor;
+			$self-> draw_item_background($canvas, 
+				$left, $bottom, $right, $top, $prelight,
+				$selected ? $self-> hiliteBackColor : $bc,
+			);
+			$self->backColor($bc);
+
+			my $c  = $canvas-> color;
+			$canvas-> color( $selected ? $self-> hiliteColor : $c );
+			$canvas-> text_out_bidi( $node-> [0]-> [0], $left, $bottom);
 			$canvas-> color( $c);
 		} else {
-			$canvas-> text_out( $node-> [0]-> [0], $left, $bottom);
+			$canvas-> text_out_bidi( $node-> [0]-> [0], $left, $bottom);
 		}
 		$canvas-> rect_focus( $left, $bottom, $right, $top) if $focused;
 	}
@@ -1764,16 +1804,20 @@ sub draw_items
 {
 	my ($self, $canvas, $paintStruc) = @_;
 	for ( @$paintStruc) {
-		my ( $node, $left, $bottom, $right, $top, $position, $selected, $focused) = @$_;
+		my ( $node, $left, $bottom, $right, $top, $position, $selected, $focused, $prelight) = @$_;
 		my $c;
 		my $dw = length $node-> [0]-> [1] ?
 			$self-> {iconSizes}-> [0] :
 			$node-> [0]-> [2];
-		if ( $selected) {
+		if ( $selected || $prelight) {
 			$c = $canvas-> color;
-			$canvas-> color( $self-> hiliteBackColor);
-			$canvas-> bar( $left - $self-> {indent} / 4, $bottom, $right, $top);
-			$canvas-> color( $self-> hiliteColor);
+			my $bc = $self->backColor;
+			$self-> draw_item_background($canvas, 
+				$left - $self-> {indent} / 4, $bottom, $right, $top, $prelight,
+				$selected ? $self-> hiliteBackColor : $self-> backColor
+			);
+			$self->backColor($bc);
+			$canvas-> color( $selected ? $self-> hiliteColor : $c );
 		}
 		my $icon = (length( $node-> [0]-> [1]) || $unix) ?
 			( $node-> [2] ? $self-> {openedIcon} : $self-> {closedIcon}) : 
@@ -1783,12 +1827,12 @@ sub draw_items
 			int($bottom + ( $self-> {itemHeight} - $self-> {iconSizes}-> [1]) / 2),
 			$icon
 		);
-		$canvas-> text_out( 
+		$canvas-> text_out_bidi( 
 			$node-> [0]-> [0], 
 			$left + $dw,
 			int($bottom + ( $self-> {itemHeight} - $self-> {fontHeight}) / 2)
 		);
-		$canvas-> color( $c) if $selected;
+		$canvas-> color( $c) if $selected || $prelight;
 		$canvas-> rect_focus( $left - $self-> {indent} / 4, $bottom, $right, $top) 
 			if $focused;
 	}
@@ -2076,23 +2120,23 @@ with the following definitions of indices:
 
 =over
 
-=item 0
+=item #0
 
 Item id with non-defined format. The simplest implementation, C<Prima::StringOutline>, 
 treats the scalar as a text string. The more complex classes store 
 references to arrays or hashes here. See C<items> article of a concrete class
 for the format of a node record.
 
-=item 1
+=item #1
 
 Reference to a child node. C<undef> if there is none.
 
-=item 2
+=item #2
 
 A boolean flag, which selects if the node shown as expanded, e.g.
 all its immediate children are visible.
 
-=item 3
+=item #3
 
 Width of an item in pixels.
 
@@ -2334,32 +2378,32 @@ ACTION subroutine is called with the following parameters:
 
 =over
 
-=item 0
+=item #0
 
 Node reference
 
-=item 1
+=item #1
 
 Parent node reference; if C<undef>, the node is the root.
 
-=item 2
+=item #2
 
 Node offset in parent item list.
 
-=item 3
+=item #3
 
 Node index.
 
-=item 4
+=item #4
 
 Node depth level. 0 means the root node.
 
-=item 5
+=item #5
 
 A boolean flag, set to 1 if the node is the last child in parent node list,
 set to 0 otherwise.
 
-=item 6
+=item #6
 
 Visibility index. When C<iterate> is called with C<FULL = 1>, the index is
 the item index as seen of the screen. If the item is not visible, the index
@@ -2451,12 +2495,12 @@ Called when the user finishes the drag of an item
 from OLD_INDEX to NEW_INDEX position. The default action
 rearranges the item list in accord with the dragging action.
 
-=item DrawItem CANVAS, NODE, X1, Y1, X2, Y2, INDEX, SELECTED, FOCUSED
+=item DrawItem CANVAS, NODE, X1, Y1, X2, Y2, INDEX, SELECTED, FOCUSED, PRELIGHT
 
 Called when INDEXth item, contained in NODE is to be drawn on 
 CANVAS. X1, Y1, X2, Y2 coordinated define the exterior rectangle
-of the item in widget coordinates. SELECTED and FOCUSED boolean flags are set to
-1 if the item is selected or focused, respectively; 0 otherwise.
+of the item in widget coordinates. SELECTED, FOCUSED, and PRELIGHT boolean flags are set to
+1 if the item is selected, focused, or pre-lighted, respectively; 0 otherwise.
 
 =item MeasureItem NODE, LEVEL, REF
 
@@ -2509,19 +2553,19 @@ The format of a node is defined as follows:
 
 =over
 
-=item 0
+=item #0
 
 Directory name, string.
 
-=item 1
+=item #1
 
 Parent path; an empty string for the root items.
 
-=item 2
+=item #2
 
 Icon width in pixels, integer.
 
-=item 3
+=item #3
 
 Drive icon; defined only for the root items under non-unix hosts
 in order to reflect the drive type ( hard, floppy, etc ).
